use actix::prelude::*;
use crate::server::server::GameConnection;
use crate::server::gamelobby::GameLobby;
use crate::server::protocol::*;

#[derive(Message)]
#[rtype(result = "()")]
pub struct JoinRequest {
    pub lobby_id: String,
    pub nick: String,
    pub p: Addr<GameConnection>
}

#[derive(Message)]
#[rtype(result = "()")]
pub struct CreateLobbyRequest {
    pub lobby_id: String,
    pub nick: String,
    pub p: Addr<GameConnection>
}

#[derive(Message)]
#[rtype(result = "()")]
pub struct ReadyMsg(pub String);

#[derive(Message)]
#[rtype(result = "()")]
pub struct UnreadyMsg(pub String);

#[derive(Message)]
#[rtype(result = "()")]
pub struct RevealMsg(pub String);

#[derive(Message)]
#[rtype(result = "()")]
pub struct LeaveMsg(pub String);

#[derive(Message)]
#[rtype(result = "()")]
pub struct LobbyFinished(pub String);

#[derive(Message)]
#[rtype(result = "()")]
pub struct StopMsg;

#[derive(Message)]
#[rtype(result = "()")]
pub struct SubmitWordMsg {
    pub word: String,
    pub nick: String
}

#[derive(Message)]
#[rtype(result = "()")]
pub struct SubmitGuessMsg {
    pub guesses: Vec<(String, String)>,
    pub nick: String
}

#[derive(Message)]
#[rtype(result = "()")]
pub struct ResultMsg {
    pub results: RoundResultData
}

#[derive(Message)]
#[rtype(result = "()")]
pub struct NoSuchLobby(pub String);

#[derive(Message)]
#[rtype(result = "()")]
pub struct LobbyJoined {
    pub lobby: Addr<GameLobby>,
    pub game_id: String,
    pub nick: String
}

#[derive(Message)]
#[rtype(result = "()")]
pub struct GameUpdate {
    pub game_data: GameData
}

#[derive(Message)]
#[rtype(result = "()")]
pub struct PlayerListUpdate {
    pub player_list: Vec<PlayerData>
}
